@@ -97,7 +97,7 @@ class ShortRepository(object):
             parts = parts[:-1]
         else:
             tail = b''
-        context = dict((b'%d' % (i + 1), v) for i, v in enumerate(parts))
+        context = {b'%d' % (i + 1): v for i, v in enumerate(parts)}
         return b''.join(self.templater.process(self.url, context)) + tail
 
 
@@ -142,8 +142,7 @@ def extsetup(ui):
 
 @command(b'debugexpandscheme', norepo=True)
 def expandscheme(ui, url, **opts):
-    """given a repo path, provide the scheme-expanded path
-    """
+    """given a repo path, provide the scheme-expanded path"""
     repo = hg._peerlookup(url)
     if isinstance(repo, ShortRepository):
         url = repo.resolve(url)
