@@ -316,8 +316,8 @@ class server(object):
         return -1
 
     def runcommand(self):
-        """ reads a list of \0 terminated arguments, executes
-        and writes the return code to the result channel """
+        """reads a list of \0 terminated arguments, executes
+        and writes the return code to the result channel"""
         from . import dispatch  # avoid cycle
 
         args = self._readlist()
@@ -500,7 +500,7 @@ def _serverequest(ui, repo, conn, createcmdserver, prereposetups):
         # handle exceptions that may be raised by command server. most of
         # known exceptions are caught by dispatch.
         except error.Abort as inst:
-            ui.error(_(b'abort: %s\n') % inst)
+            ui.error(_(b'abort: %s\n') % inst.message)
         except IOError as inst:
             if inst.errno != errno.EPIPE:
                 raise
