@@ -20,6 +20,7 @@ Data depends on type.
 from __future__ import absolute_import
 
 from .node import nullrev
+from .thirdparty import attr
 from . import (
     dagop,
     pycompat,
@@ -27,15 +28,16 @@ from . import (
     util,
 )
 
-CHANGESET = 'C'
-PARENT = 'P'
-GRANDPARENT = 'G'
-MISSINGPARENT = 'M'
+CHANGESET = b'C'
+PARENT = b'P'
+GRANDPARENT = b'G'
+MISSINGPARENT = b'M'
 # Style of line to draw. None signals a line that ends and is removed at this
 # point. A number prefix means only the last N characters of the current block
 # will use that style, the rest will use the PARENT style. Add a - sign
 # (so making N negative) and all but the first N characters use that style.
-EDGES = {PARENT: '|', GRANDPARENT: ':', MISSINGPARENT: None}
+EDGES = {PARENT: b'|', GRANDPARENT: b':', MISSINGPARENT: None}
+
 
 def dagwalker(repo, revs):
     """cset DAG generator yielding (id, CHANGESET, ctx, [parentinfo]) tuples
@@ -56,9 +58,12 @@ def dagwalker(repo, revs):
         # partition into parents in the rev set and missing parents, then
         # augment the lists with markers, to inform graph drawing code about
         # what kind of edge to draw between nodes.
-        pset = set(p.rev() for p in ctx.parents() if p.rev() in revs)
-        mpars = [p.rev() for p in ctx.parents()
-                 if p.rev() != nullrev and p.rev() not in pset]
+        pset = {p.rev() for p in ctx.parents() if p.rev() in revs}
+        mpars = [
+            p.rev()
+            for p in ctx.parents()
+            if p.rev() != nullrev and p.rev() not in pset
+        ]
         parents = [(PARENT, p) for p in sorted(pset)]
 
         for mpar in mpars:
@@ -68,8 +73,9 @@ def dagwalker(repo, revs):
                 # through all revs (issue4782)
                 if not isinstance(revs, smartset.baseset):
                     revs = smartset.baseset(revs)
-                gp = gpcache[mpar] = sorted(set(dagop.reachableroots(
-                    repo, revs, [mpar])))
+                gp = gpcache[mpar] = sorted(
+                    set(dagop.reachableroots(repo, revs, [mpar]))
+                )
             if not gp:
                 parents.append((MISSINGPARENT, mpar))
                 pset.add(mpar)
@@ -79,6 +85,7 @@ def dagwalker(repo, revs):
 
         yield (ctx.rev(), CHANGESET, ctx, parents)
 
+
 def nodes(repo, nodes):
     """cset DAG generator yielding (id, CHANGESET, ctx, [parentids]) tuples
 
@@ -88,10 +95,12 @@ def nodes(repo, nodes):
     include = set(nodes)
     for node in nodes:
         ctx = repo[node]
-        parents = set((PARENT, p.rev()) for p in ctx.parents()
-                      if p.node() in include)
+        parents = {
+            (PARENT, p.rev()) for p in ctx.parents() if p.node() in include
+        }
         yield (ctx.rev(), CHANGESET, ctx, sorted(parents))
 
+
 def colored(dag, repo):
     """annotates a DAG with colored edge information
 
@@ -110,18 +119,19 @@ def colored(dag, repo):
     newcolor = 1
     config = {}
 
-    for key, val in repo.ui.configitems('graph'):
-        if '.' in key:
-            branch, setting = key.rsplit('.', 1)
+    for key, val in repo.ui.configitems(b'graph'):
+        if b'.' in key:
+            branch, setting = key.rsplit(b'.', 1)
             # Validation
-            if setting == "width" and val.isdigit():
+            if setting == b"width" and val.isdigit():
                 config.setdefault(branch, {})[setting] = int(val)
-            elif setting == "color" and val.isalnum():
+            elif setting == b"color" and val.isalnum():
                 config.setdefault(branch, {})[setting] = val
 
     if config:
         getconf = util.lrucachefunc(
-            lambda rev: config.get(repo[rev].branch(), {}))
+            lambda rev: config.get(repo[rev].branch(), {})
+        )
     else:
         getconf = lambda rev: {}
 
@@ -129,7 +139,7 @@ def colored(dag, repo):
 
         # Compute seen and next
         if cur not in seen:
-            seen.append(cur) # new head
+            seen.append(cur)  # new head
             colors[cur] = newcolor
             newcolor += 1
 
@@ -139,7 +149,7 @@ def colored(dag, repo):
 
         # Add parents to next
         addparents = [p for pt, p in parents if p not in next]
-        next[col:col + 1] = addparents
+        next[col : col + 1] = addparents
 
         # Set colors for the parents
         for i, p in enumerate(addparents):
@@ -154,25 +164,36 @@ def colored(dag, repo):
         for ecol, eid in enumerate(seen):
             if eid in next:
                 bconf = getconf(eid)
-                edges.append((
-                    ecol, next.index(eid), colors[eid],
-                    bconf.get('width', -1),
-                    bconf.get('color', '')))
+                edges.append(
+                    (
+                        ecol,
+                        next.index(eid),
+                        colors[eid],
+                        bconf.get(b'width', -1),
+                        bconf.get(b'color', b''),
+                    )
+                )
             elif eid == cur:
                 for ptype, p in parents:
                     bconf = getconf(p)
-                    edges.append((
-                        ecol, next.index(p), color,
-                        bconf.get('width', -1),
-                        bconf.get('color', '')))
+                    edges.append(
+                        (
+                            ecol,
+                            next.index(p),
+                            color,
+                            bconf.get(b'width', -1),
+                            bconf.get(b'color', b''),
+                        )
+                    )
 
         # Yield and move on
         yield (cur, type, data, (col, color), edges)
         seen = next
 
+
 def asciiedges(type, char, state, rev, parents):
     """adds edge info to changelog DAG walk suitable for ascii()"""
-    seen = state['seen']
+    seen = state.seen
     if rev not in seen:
         seen.append(rev)
     nodeidx = seen.index(rev)
@@ -187,12 +208,12 @@ def asciiedges(type, char, state, rev, parents):
             knownparents.append(parent)
         else:
             newparents.append(parent)
-            state['edges'][parent] = state['styles'].get(ptype, '|')
+            state.edges[parent] = state.styles.get(ptype, b'|')
 
     ncols = len(seen)
     width = 1 + ncols * 2
     nextseen = seen[:]
-    nextseen[nodeidx:nodeidx + 1] = newparents
+    nextseen[nodeidx : nodeidx + 1] = newparents
     edges = [(nodeidx, nextseen.index(p)) for p in knownparents]
 
     seen[:] = nextseen
@@ -206,7 +227,7 @@ def asciiedges(type, char, state, rev, parents):
         nmorecols = 1
         width += 2
         yield (type, char, width, (nodeidx, edges, ncols, nmorecols))
-        char = '\\'
+        char = b'\\'
         nodeidx += 1
         ncols += 1
         edges = []
@@ -220,50 +241,53 @@ def asciiedges(type, char, state, rev, parents):
     if nmorecols > 0:
         width += 2
     # remove current node from edge characters, no longer needed
-    state['edges'].pop(rev, None)
+    state.edges.pop(rev, None)
     yield (type, char, width, (nodeidx, edges, ncols, nmorecols))
 
+
 def _fixlongrightedges(edges):
     for (i, (start, end)) in enumerate(edges):
         if end > start:
             edges[i] = (start, end + 1)
 
-def _getnodelineedgestail(
-        echars, idx, pidx, ncols, coldiff, pdiff, fix_tail):
+
+def _getnodelineedgestail(echars, idx, pidx, ncols, coldiff, pdiff, fix_tail):
     if fix_tail and coldiff == pdiff and coldiff != 0:
         # Still going in the same non-vertical direction.
         if coldiff == -1:
             start = max(idx + 1, pidx)
-            tail = echars[idx * 2:(start - 1) * 2]
-            tail.extend(["/", " "] * (ncols - start))
+            tail = echars[idx * 2 : (start - 1) * 2]
+            tail.extend([b"/", b" "] * (ncols - start))
             return tail
         else:
-            return ["\\", " "] * (ncols - idx - 1)
+            return [b"\\", b" "] * (ncols - idx - 1)
     else:
-        remainder = (ncols - idx - 1)
-        return echars[-(remainder * 2):] if remainder > 0 else []
+        remainder = ncols - idx - 1
+        return echars[-(remainder * 2) :] if remainder > 0 else []
+
 
 def _drawedges(echars, edges, nodeline, interline):
     for (start, end) in edges:
         if start == end + 1:
-            interline[2 * end + 1] = "/"
+            interline[2 * end + 1] = b"/"
         elif start == end - 1:
-            interline[2 * start + 1] = "\\"
+            interline[2 * start + 1] = b"\\"
         elif start == end:
             interline[2 * start] = echars[2 * start]
         else:
             if 2 * end >= len(nodeline):
                 continue
-            nodeline[2 * end] = "+"
+            nodeline[2 * end] = b"+"
             if start > end:
                 (start, end) = (end, start)
             for i in range(2 * start + 1, 2 * end):
-                if nodeline[i] != "+":
-                    nodeline[i] = "-"
+                if nodeline[i] != b"+":
+                    nodeline[i] = b"-"
+
 
 def _getpaddingline(echars, idx, ncols, edges):
     # all edges up to the current node
-    line = echars[:idx * 2]
+    line = echars[: idx * 2]
     # an edge for the current node, if there is one
     if (idx, idx - 1) in edges or (idx, idx) in edges:
         # (idx, idx - 1)      (idx, idx)
@@ -272,15 +296,16 @@ def _getpaddingline(echars, idx, ncols, edges):
         # | | X |           | X | |
         # | |/ /            | |/ /
         # | | |             | | |
-        line.extend(echars[idx * 2:(idx + 1) * 2])
+        line.extend(echars[idx * 2 : (idx + 1) * 2])
     else:
-        line.extend([' ', ' '])
+        line.extend([b' ', b' '])
     # all edges to the right of the current node
     remainder = ncols - idx - 1
     if remainder > 0:
-        line.extend(echars[-(remainder * 2):])
+        line.extend(echars[-(remainder * 2) :])
     return line
 
+
 def _drawendinglines(lines, extra, edgemap, seen, state):
     """Draw ending lines for missing parent edges
 
@@ -298,7 +323,7 @@ def _drawendinglines(lines, extra, edgemap, seen, state):
     while edgechars and edgechars[-1] is None:
         edgechars.pop()
     shift_size = max((edgechars.count(None) * 2) - 1, 0)
-    minlines = 3 if not state['graphshorten'] else 2
+    minlines = 3 if not state.graphshorten else 2
     while len(lines) < minlines + shift_size:
         lines.append(extra[:])
 
@@ -314,17 +339,17 @@ def _drawendinglines(lines, extra, edgemap, seen, state):
         targets = list(range(first_empty, first_empty + len(toshift) * 2, 2))
         positions = toshift[:]
         for line in lines[-shift_size:]:
-            line[first_empty:] = [' '] * (len(line) - first_empty)
+            line[first_empty:] = [b' '] * (len(line) - first_empty)
             for i in range(len(positions)):
                 pos = positions[i] - 1
                 positions[i] = max(pos, targets[i])
-                line[pos] = '/' if pos > targets[i] else extra[toshift[i]]
+                line[pos] = b'/' if pos > targets[i] else extra[toshift[i]]
 
-    map = {1: '|', 2: '~'} if not state['graphshorten'] else {1: '~'}
+    map = {1: b'|', 2: b'~'} if not state.graphshorten else {1: b'~'}
     for i, line in enumerate(lines):
         if None not in line:
             continue
-        line[:] = [c or map.get(i, ' ') for c in line]
+        line[:] = [c or map.get(i, b' ') for c in line]
 
     # remove edges that ended
     remove = [p for p, c in edgemap.items() if c is None]
@@ -332,16 +357,18 @@ def _drawendinglines(lines, extra, edgemap, seen, state):
         del edgemap[parent]
         seen.remove(parent)
 
-def asciistate():
-    """returns the initial value for the "state" argument to ascii()"""
-    return {
-        'seen': [],
-        'edges': {},
-        'lastcoldiff': 0,
-        'lastindex': 0,
-        'styles': EDGES.copy(),
-        'graphshorten': False,
-    }
+
+@attr.s
+class asciistate(object):
+    """State of ascii() graph rendering"""
+
+    seen = attr.ib(init=False, default=attr.Factory(list))
+    edges = attr.ib(init=False, default=attr.Factory(dict))
+    lastcoldiff = attr.ib(init=False, default=0)
+    lastindex = attr.ib(init=False, default=0)
+    styles = attr.ib(init=False, default=attr.Factory(EDGES.copy))
+    graphshorten = attr.ib(init=False, default=False)
+
 
 def outputgraph(ui, graph):
     """outputs an ASCII graph of a DAG
@@ -357,7 +384,8 @@ def outputgraph(ui, graph):
     without needing to mimic all of the edge-fixup logic in ascii()
     """
     for (ln, logstr) in graph:
-        ui.write((ln + logstr).rstrip() + "\n")
+        ui.write((ln + logstr).rstrip() + b"\n")
+
 
 def ascii(ui, state, type, char, text, coldata):
     """prints an ASCII graph of the DAG
@@ -382,11 +410,11 @@ def ascii(ui, state, type, char, text, coldata):
     idx, edges, ncols, coldiff = coldata
     assert -2 < coldiff < 2
 
-    edgemap, seen = state['edges'], state['seen']
+    edgemap, seen = state.edges, state.seen
     # Be tolerant of history issues; make sure we have at least ncols + coldiff
     # elements to work with. See test-glog.t for broken history test cases.
-    echars = [c for p in seen for c in (edgemap.get(p, '|'), ' ')]
-    echars.extend(('|', ' ') * max(ncols + coldiff - len(seen), 0))
+    echars = [c for p in seen for c in (edgemap.get(p, b'|'), b' ')]
+    echars.extend((b'|', b' ') * max(ncols + coldiff - len(seen), 0))
 
     if coldiff == -1:
         # Transform
@@ -404,8 +432,9 @@ def ascii(ui, state, type, char, text, coldata):
     #     |  / /         |   | |  # <--- padding line
     #     o | |          |  / /
     #                    o | |
-    add_padding_line = (len(text) > 2 and coldiff == -1 and
-                        [x for (x, y) in edges if x + 1 < y])
+    add_padding_line = (
+        len(text) > 2 and coldiff == -1 and [x for (x, y) in edges if x + 1 < y]
+    )
 
     # fix_nodeline_tail says whether to rewrite
     #
@@ -417,28 +446,35 @@ def ascii(ui, state, type, char, text, coldata):
     fix_nodeline_tail = len(text) <= 2 and not add_padding_line
 
     # nodeline is the line containing the node character (typically o)
-    nodeline = echars[:idx * 2]
-    nodeline.extend([char, " "])
+    nodeline = echars[: idx * 2]
+    nodeline.extend([char, b" "])
 
     nodeline.extend(
         _getnodelineedgestail(
-            echars, idx, state['lastindex'], ncols, coldiff,
-            state['lastcoldiff'], fix_nodeline_tail))
+            echars,
+            idx,
+            state.lastindex,
+            ncols,
+            coldiff,
+            state.lastcoldiff,
+            fix_nodeline_tail,
+        )
+    )
 
     # shift_interline is the line containing the non-vertical
     # edges between this entry and the next
-    shift_interline = echars[:idx * 2]
+    shift_interline = echars[: idx * 2]
     for i in pycompat.xrange(2 + coldiff):
-        shift_interline.append(' ')
+        shift_interline.append(b' ')
     count = ncols - idx - 1
     if coldiff == -1:
         for i in pycompat.xrange(count):
-            shift_interline.extend(['/', ' '])
+            shift_interline.extend([b'/', b' '])
     elif coldiff == 0:
-        shift_interline.extend(echars[(idx + 1) * 2:ncols * 2])
+        shift_interline.extend(echars[(idx + 1) * 2 : ncols * 2])
     else:
         for i in pycompat.xrange(count):
-            shift_interline.extend(['\\', ' '])
+            shift_interline.extend([b'\\', b' '])
 
     # draw edges from the current node to its parents
     _drawedges(echars, edges, nodeline, shift_interline)
@@ -450,7 +486,7 @@ def ascii(ui, state, type, char, text, coldata):
 
     # If 'graphshorten' config, only draw shift_interline
     # when there is any non vertical flow in graph.
-    if state['graphshorten']:
+    if state.graphshorten:
         if any(c in br'\/' for c in shift_interline if c):
             lines.append(shift_interline)
     # Else, no 'graphshorten' config so draw shift_interline.
@@ -459,7 +495,7 @@ def ascii(ui, state, type, char, text, coldata):
 
     # make sure that there are as many graph lines as there are
     # log strings
-    extra_interline = echars[:(ncols + coldiff) * 2]
+    extra_interline = echars[: (ncols + coldiff) * 2]
     if len(lines) < len(text):
         while len(lines) < len(text):
             lines.append(extra_interline[:])
@@ -467,29 +503,15 @@ def ascii(ui, state, type, char, text, coldata):
     _drawendinglines(lines, extra_interline, edgemap, seen, state)
 
     while len(text) < len(lines):
-        text.append("")
-
-    if any(len(char) > 1 for char in edgemap.values()):
-        # limit drawing an edge to the first or last N lines of the current
-        # section the rest of the edge is drawn like a parent line.
-        parent = state['styles'][PARENT][-1:]
-        def _drawgp(char, i):
-            # should a grandparent character be drawn for this line?
-            if len(char) < 2:
-                return True
-            num = int(char[:-1])
-            # either skip first num lines or take last num lines, based on sign
-            return -num <= i if num < 0 else (len(lines) - i) <= num
-        for i, line in enumerate(lines):
-            line[:] = [c[-1:] if _drawgp(c, i) else parent for c in line]
-        edgemap.update(
-            (e, (c if len(c) < 2 else parent)) for e, c in edgemap.items())
+        text.append(b"")
 
     # print lines
     indentation_level = max(ncols, ncols + coldiff)
-    lines = ["%-*s " % (2 * indentation_level, "".join(line)) for line in lines]
+    lines = [
+        b"%-*s " % (2 * indentation_level, b"".join(line)) for line in lines
+    ]
     outputgraph(ui, zip(lines, text))
 
     # ... and start over
-    state['lastcoldiff'] = coldiff
-    state['lastindex'] = idx
+    state.lastcoldiff = coldiff
+    state.lastindex = idx
