@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 
+import contextlib
 import errno
 import os
 import posixpath
@@ -9,15 +10,21 @@ from .i18n import _
 from . import (
     encoding,
     error,
+    policy,
     pycompat,
     util,
 )
 
+rustdirs = policy.importrust('dirstate', 'Dirs')
+parsers = policy.importmod('parsers')
+
+
 def _lowerclean(s):
     return encoding.hfsignoreclean(s.lower())
 
+
 class pathauditor(object):
-    '''ensure that a filesystem path contains no banned components.
+    """ensure that a filesystem path contains no banned components.
     the following properties of a path are checked:
 
     - ends with a directory separator
@@ -37,7 +44,7 @@ class pathauditor(object):
     If 'cached' is set to True, audited paths and sub-directories are cached.
     Be careful to not keep the cache of unmanaged directories for long because
     audited paths may be replaced with symlinks.
-    '''
+    """
 
     def __init__(self, root, callback=None, realfs=True, cached=False):
         self.audited = set()
@@ -52,8 +59,8 @@ class pathauditor(object):
             self.normcase = lambda x: x
 
     def __call__(self, path, mode=None):
-        '''Check the relative path.
-        path may contain a pattern (e.g. foodir/**.txt)'''
+        """Check the relative path.
+        path may contain a pattern (e.g. foodir/**.txt)"""
 
         path = util.localpath(path)
         normpath = self.normcase(path)
@@ -61,51 +68,55 @@ class pathauditor(object):
             return
         # AIX ignores "/" at end of path, others raise EISDIR.
         if util.endswithsep(path):
-            raise error.Abort(_("path ends in directory separator: %s") % path)
+            raise error.Abort(_(b"path ends in directory separator: %s") % path)
         parts = util.splitpath(path)
-        if (os.path.splitdrive(path)[0]
-            or _lowerclean(parts[0]) in ('.hg', '.hg.', '')
-            or os.pardir in parts):
-            raise error.Abort(_("path contains illegal component: %s") % path)
+        if (
+            os.path.splitdrive(path)[0]
+            or _lowerclean(parts[0]) in (b'.hg', b'.hg.', b'')
+            or pycompat.ospardir in parts
+        ):
+            raise error.Abort(_(b"path contains illegal component: %s") % path)
         # Windows shortname aliases
         for p in parts:
-            if "~" in p:
-                first, last = p.split("~", 1)
-                if last.isdigit() and first.upper() in ["HG", "HG8B6C"]:
-                    raise error.Abort(_("path contains illegal component: %s")
-                                     % path)
-        if '.hg' in _lowerclean(path):
-            lparts = [_lowerclean(p.lower()) for p in parts]
-            for p in '.hg', '.hg.':
+            if b"~" in p:
+                first, last = p.split(b"~", 1)
+                if last.isdigit() and first.upper() in [b"HG", b"HG8B6C"]:
+                    raise error.Abort(
+                        _(b"path contains illegal component: %s") % path
+                    )
+        if b'.hg' in _lowerclean(path):
+            lparts = [_lowerclean(p) for p in parts]
+            for p in b'.hg', b'.hg.':
                 if p in lparts[1:]:
                     pos = lparts.index(p)
                     base = os.path.join(*parts[:pos])
-                    raise error.Abort(_("path '%s' is inside nested repo %r")
-                                     % (path, base))
+                    raise error.Abort(
+                        _(b"path '%s' is inside nested repo %r")
+                        % (path, pycompat.bytestr(base))
+                    )
 
         normparts = util.splitpath(normpath)
         assert len(parts) == len(normparts)
 
         parts.pop()
         normparts.pop()
-        prefixes = []
         # It's important that we check the path parts starting from the root.
-        # This means we won't accidentally traverse a symlink into some other
-        # filesystem (which is potentially expensive to access).
+        # We don't want to add "foo/bar/baz" to auditeddir before checking if
+        # there's a "foo/.hg" directory. This also means we won't accidentally
+        # traverse a symlink into some other filesystem (which is potentially
+        # expensive to access).
         for i in range(len(parts)):
-            prefix = pycompat.ossep.join(parts[:i + 1])
-            normprefix = pycompat.ossep.join(normparts[:i + 1])
+            prefix = pycompat.ossep.join(parts[: i + 1])
+            normprefix = pycompat.ossep.join(normparts[: i + 1])
             if normprefix in self.auditeddir:
                 continue
             if self._realfs:
                 self._checkfs(prefix, path)
-            prefixes.append(normprefix)
+            if self._cached:
+                self.auditeddir.add(normprefix)
 
         if self._cached:
             self.audited.add(normpath)
-            # only add prefixes to the cache after checking everything: we don't
-            # want to add "foo/bar/baz" before checking if there's a "foo/.hg"
-            self.auditeddir.update(prefixes)
 
     def _checkfs(self, prefix, path):
         """raise exception if a file system backed check fails"""
@@ -119,13 +130,17 @@ class pathauditor(object):
                 raise
         else:
             if stat.S_ISLNK(st.st_mode):
-                msg = _('path %r traverses symbolic link %r') % (path, prefix)
+                msg = _(b'path %r traverses symbolic link %r') % (
+                    pycompat.bytestr(path),
+                    pycompat.bytestr(prefix),
+                )
                 raise error.Abort(msg)
-            elif (stat.S_ISDIR(st.st_mode) and
-                  os.path.isdir(os.path.join(curpath, '.hg'))):
+            elif stat.S_ISDIR(st.st_mode) and os.path.isdir(
+                os.path.join(curpath, b'.hg')
+            ):
                 if not self.callback or not self.callback(curpath):
-                    msg = _("path '%s' is inside nested repo %r")
-                    raise error.Abort(msg % (path, prefix))
+                    msg = _(b"path '%s' is inside nested repo %r")
+                    raise error.Abort(msg % (path, pycompat.bytestr(prefix)))
 
     def check(self, path):
         try:
@@ -134,8 +149,22 @@ class pathauditor(object):
         except (OSError, error.Abort):
             return False
 
+    @contextlib.contextmanager
+    def cached(self):
+        if self._cached:
+            yield
+        else:
+            try:
+                self._cached = True
+                yield
+            finally:
+                self.audited.clear()
+                self.auditeddir.clear()
+                self._cached = False
+
+
 def canonpath(root, cwd, myname, auditor=None):
-    '''return the canonical path of myname, given cwd and root
+    """return the canonical path of myname, given cwd and root
 
     >>> def check(root, cwd, myname):
     ...     a = pathauditor(root, realfs=False)
@@ -175,7 +204,7 @@ def canonpath(root, cwd, myname, auditor=None):
     'filename'
     >>> unixonly(b'/repo', b'/repo/subdir', b'filename', b'subdir/filename')
     'subdir/filename'
-    '''
+    """
     if util.endswithsep(root):
         rootsep = root
     else:
@@ -187,11 +216,11 @@ def canonpath(root, cwd, myname, auditor=None):
     if auditor is None:
         auditor = pathauditor(root)
     if name != rootsep and name.startswith(rootsep):
-        name = name[len(rootsep):]
+        name = name[len(rootsep) :]
         auditor(name)
         return util.pconvert(name)
     elif name == root:
-        return ''
+        return b''
     else:
         # Determine whether `name' is in the hierarchy at or beneath `root',
         # by iterating name=dirname(name) until that causes no change (can't
@@ -207,7 +236,7 @@ def canonpath(root, cwd, myname, auditor=None):
             if s:
                 if not rel:
                     # name was actually the same as root (maybe a symlink)
-                    return ''
+                    return b''
                 rel.reverse()
                 name = os.path.join(*rel)
                 auditor(name)
@@ -224,18 +253,20 @@ def canonpath(root, cwd, myname, auditor=None):
         try:
             if cwd != root:
                 canonpath(root, root, myname, auditor)
-                relpath = util.pathto(root, cwd, '')
-                if relpath[-1] == pycompat.ossep:
+                relpath = util.pathto(root, cwd, b'')
+                if relpath.endswith(pycompat.ossep):
                     relpath = relpath[:-1]
-                hint = (_("consider using '--cwd %s'") % relpath)
+                hint = _(b"consider using '--cwd %s'") % relpath
         except error.Abort:
             pass
 
-        raise error.Abort(_("%s not under root '%s'") % (myname, root),
-                         hint=hint)
+        raise error.Abort(
+            _(b"%s not under root '%s'") % (myname, root), hint=hint
+        )
+
 
 def normasprefix(path):
-    '''normalize the specified path as path prefix
+    """normalize the specified path as path prefix
 
     Returned value can be used safely for "p.startswith(prefix)",
     "p[len(prefix):]", and so on.
@@ -249,13 +280,77 @@ def normasprefix(path):
     '/foo/bar/'
     >>> normasprefix(b'/').replace(pycompat.ossep, b'/')
     '/'
-    '''
+    """
     d, p = os.path.splitdrive(path)
     if len(p) != len(pycompat.ossep):
         return path + pycompat.ossep
     else:
         return path
 
+
+def finddirs(path):
+    pos = path.rfind(b'/')
+    while pos != -1:
+        yield path[:pos]
+        pos = path.rfind(b'/', 0, pos)
+    yield b''
+
+
+class dirs(object):
+    '''a multiset of directory names from a set of file paths'''
+
+    def __init__(self, map, skip=None):
+        """
+        a dict map indicates a dirstate while a list indicates a manifest
+        """
+        self._dirs = {}
+        addpath = self.addpath
+        if isinstance(map, dict) and skip is not None:
+            for f, s in pycompat.iteritems(map):
+                if s[0] != skip:
+                    addpath(f)
+        elif skip is not None:
+            raise error.ProgrammingError(
+                b"skip character is only supported with a dict source"
+            )
+        else:
+            for f in map:
+                addpath(f)
+
+    def addpath(self, path):
+        dirs = self._dirs
+        for base in finddirs(path):
+            if base.endswith(b'/'):
+                raise ValueError(
+                    "found invalid consecutive slashes in path: %r" % base
+                )
+            if base in dirs:
+                dirs[base] += 1
+                return
+            dirs[base] = 1
+
+    def delpath(self, path):
+        dirs = self._dirs
+        for base in finddirs(path):
+            if dirs[base] > 1:
+                dirs[base] -= 1
+                return
+            del dirs[base]
+
+    def __iter__(self):
+        return iter(self._dirs)
+
+    def __contains__(self, d):
+        return d in self._dirs
+
+
+if util.safehasattr(parsers, 'dirs'):
+    dirs = parsers.dirs
+
+if rustdirs is not None:
+    dirs = rustdirs
+
+
 # forward two methods from posixpath that do what we need, but we'd
 # rather not let our internals know that we're thinking in posix terms
 # - instead we'll let them be oblivious.
